# C1010.tcl---C1010 module control frame



set frame "C1010"
set frametitle "C1010 VME/FERA Timing Controller"
set icontitle "C1010"

set modulename  c1010
set moduletype  C1010

set defaultmodulelist {1}
set modulelist {???}
set moduleindex 0

set maxmodule 1

set maxmodule [inform vmecontrol inquire [set modulename]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [lindex $modulelist $moduleindex]

set allmodules 0


set w2r(RUN)        "run"
set w2r(STATUS)     "status"
set w2r(CONTROL)    "control"
set w2r(OPT)        "opt"

set ch_count 0
set clk_div 0



loadsource common.vme


proc click-widget {w v} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $v
    } else {
        write-vme-registers $modulename#\[?,??\].$register $v
    }

    click-redisplay
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-select-adc {v} {}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {}

proc click-redisplay {} {
   global w2r modulename module maxmodule ch_count clk_div

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   set register $w2r(STATUS)
   :modify STATUS :value [read-vme-register $modulename#$module.$register]

   set register $w2r(RUN)
   :modify RUN :value [expr [read-vme-register $modulename#$module.$register] / 4]

   set register $w2r(OPT)
   set opt [read-vme-register $modulename#$module.$register]

   set ch_count [expr $opt >> 2]
   set clk_div [expr $opt & 3]

   :modify CHCOUNT :value $ch_count
   :modify CLKDIV  :value $clk_div

}

proc start args {
   global started EGreport modulename
   if {$started == 0} then {
       make-frame
       set started 1
       EGbindings
   }
   open-frame
   click-redisplay
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug ch_count clk_div
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}

      RUN          {click-widget $w [expr $v*4]}
      CONTROL      {click-widget $w $v}

      CHCOUNT      {set ch_count $v; click-widget OPT [expr ($ch_count << 2) + $clk_div]}
      CLKDIV       {set clk_div $v;  click-widget OPT [expr ($ch_count << 2) + $clk_div]}

      ENGINEERING       {click-$v}
      REDISPLAY  	{click-redisplay}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env actionlist
  global module maxmodule modulelist maxadc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 50
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T
     :text CONTROL     :label "Control Register"  :w 6
     :choice STATUS    :label "Status"  :strings [list "clear set" "stop latch" "running"]  :deaf T :layout H :exclusive F

     :next-row T
     :choice RUN       :label "Timer"   :strings [list Halt Run] 
     :text CHCOUNT     :label "Channel Count"   :w 6  
     :choice CLKDIV    :label "Clock Divisor" :strings [list 1ms 10ms 100ms 1ms]
    
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY

      :menubutton ENGINEERING :label "Engineering"  :menu [list software] 
 
  :show T

}
